@@ -50,20 +50,22 @@ module Ole # :nodoc:
 				raise ArgumentError, 'unable to specify mode string with io object' if mode
 				[false, arg]
 			end
-			# do we have this file opened for writing? don't know of a better way to tell
-			# (unless we parse the mode string in the open case)
-			# hmmm, note that in ruby 1.9 this doesn't work anymore. which is all the more
-			# reason to use mode string parsing when available, and fall back to something like
-			# io.writeable? otherwise.
+			# force encoding, to avoid picking up source encoding with StringIO or files in text mode
+			@io.set_encoding Encoding::ASCII_8BIT if @io.respond_to?(:set_encoding)
+			# do we have this file opened for writing? use mode when provided,
+			# otherwise try no-op methods which will raise if read-only
 			@writeable = begin
 				if mode
-					IO::Mode.new(mode).writeable?
+					IOMode.new(mode).writeable?
 				else
+					# works on mri 1.8 & jruby
 					@io.flush
-					# this is for the benefit of ruby-1.9
-					# generates warnings on jruby though... :/
-					if RUBY_PLATFORM != 'java' and @io.respond_to?(:syswrite)
-						@io.syswrite('')
+					begin
+						# works on mri 1.9 & rubinius, throws EBADF on windows
+						@io.write_nonblock('') if @io.respond_to?(:write_nonblock)
+					rescue Errno::EBADF
+						# for windows
+						@io.syswrite('');
 					end
 					true
 				end
@@ -135,7 +137,7 @@ module Ole # :nodoc:
 				def to_tree idx=0
 					return [] if idx == Dirent::EOT
 					d = self[idx]
-					d.children = to_tree d.child
+					to_tree(d.child).each { |child| d << child }
 					raise FormatError, "directory #{d.inspect} used twice" if d.idx
 					d.idx = idx
 					to_tree(d.prev) + [d] + to_tree(d.next)
@@ -229,7 +231,6 @@ module Ole # :nodoc:
 			# delete a lot of stuff, and free up trailing blocks, the file size never shrinks. this can
 			# be fixed easily, add an io truncate
 			@bbat.truncate!
-			before = @io.size
 			@io.truncate @bbat.block_size * (@bbat.length + 1)
 			while true
 				# get total bbat size. equivalent to @bbat.to_s.length, but for the factoring in of
@@ -390,7 +391,7 @@ module Ole # :nodoc:
 				# 3 for this value. 
 				# transacting_signature != "\x00" * 4 or
 				if threshold != 4096 or
-					 num_mbat == 0 && mbat_start != AllocationTable::EOC or
+					 num_mbat == 0 && ![AllocationTable::EOC, AllocationTable::AVAIL].include?(mbat_start) or
 					 reserved != "\x00" * 6
 					Log.warn "may not be a valid OLE2 structured storage file"
 				end
@@ -786,14 +787,6 @@ module Ole # :nodoc:
 				@name = name
 			end
 
-			# needed to add reset of @name_lookup here. might be better to just
-			# remove this method as its only used in one place anyway.
-			def children= children
-				@children = []
-				@name_lookup = {}
-				children.each { |child| self << child }
-			end
-
 			def open mode='r'
 				raise Errno::EISDIR unless file?
 				io = RangesIOMigrateable.new self, mode
@@ -840,7 +833,7 @@ module Ole # :nodoc:
 			end
 
 			def each_child(&block)
-				@children.each(&block)
+				@children.each(&block) if dir?
 			end
 
 			# flattens the tree starting from here into +dirents+. note it modifies its argument.
